using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;

namespace Kolikkogoblin;
/// @author  Janne Yli-Tainio
/// @version 18.10.2023
/// <summary>
/// Goblin pyrkii keräämään kaikki kolikot
/// </summary>
public class Kolikkogoblin : PhysicsGame
{
    private static readonly String[] lines = {
                  "     p                  ",
                  "                        ",
                  "    *                 m ",
                  "    X   *       *   XXXX",
                  "        XX      X       ",
                  "             X          ",    
                  "       X                ",
                  "    *                   ",
                  "    X                   ",
                  "                        ",
                  "      X   X    XX  *    ",
                  "                   X    ",
                  "                 X      ",
                  "   *           X        ",
                  "  XXX        X          ",
                  "        X               ",
                  "      wwww    g         ",
                  };
    
    private static readonly int tileWidth = 800 / lines[0].Length;
    private static readonly int tileHeight = 480 / lines.Length;

    public override void Begin()
    {
        Gravity = new Vector(0, -500);
        
        Image taustakuva = LoadImage("tasonkuva1");
        Level.Background.Image = taustakuva;
        
        TileMap tiles = TileMap.FromStringArray(lines);

        tiles.SetTileMethod('X', LuoPalikka, Color.Wheat);
        tiles.SetTileMethod('*', LuoKolikko, Color.Yellow);
        tiles.SetTileMethod('g', LuoGoblin, Color.Green);
        tiles.SetTileMethod('w', LuoPiikkeja, Color.DarkGray);
        tiles.SetTileMethod('m', LuoMaali, Color.Blue);
        tiles.SetTileMethod('p', LuoPistelaskuri, Color.Blue);
        
        tiles.Execute(tileWidth, tileHeight);

        Level.CreateBorders();
        Camera.ZoomToLevel();
        
        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }


    private void LuoPalikka(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject palikka = new PhysicsObject(leveys , korkeus / 2);
        palikka.Position = paikka;
        palikka.Color = vari;
        palikka.Tag = "rakenne";
        palikka.Image = LoadImage("palikka");
        palikka.IgnoresPhysicsLogics = true;
        palikka.Mass = 100000;
        Add(palikka);
    }

    private void LuoKolikko(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject kolikko = new PhysicsObject(leveys / 2, leveys / 2, Shape.Circle);
        kolikko.Position = paikka;
        kolikko.Color = vari;
        kolikko.Tag = "kolikko";
        kolikko.Image = LoadImage("kolikko");
        kolikko.IgnoresGravity = true;
        Add(kolikko);
    }
    
    private void LuoGoblin(Vector paikka, double leveys, double korkeus, Color vari)
    { 
        PlatformCharacter goblin = new PlatformCharacter(leveys*1.5 , korkeus *1.5);
        goblin.Position = paikka;
        goblin.Color = vari;
        goblin.Image = LoadImage("goblin1");
        Add(goblin);
        
        Image[] goblinKavely = LoadImages( "goblin2");
        Image[] goblinHyppy = LoadImages( "goblin3");
        Image[] goblinPaikallaan = LoadImages( "goblin1");
        
        goblin.AnimWalk = new Animation(goblinKavely);
        goblin.AnimIdle = new Animation(goblinPaikallaan);
        goblin.AnimJump = new Animation(goblinHyppy);
        
        Keyboard.Listen(Key.Left, ButtonState.Down, LiikutaPelaajaaVasemmalle, null, new Vector(-30000, 0));
        Keyboard.Listen(Key.Right, ButtonState.Down, LiikutaPelaajaaOikealle, null, new Vector(30000, 0));
        Keyboard.Listen(Key.Up, ButtonState.Down, PelaajaHyppaa, null, new Vector(0, 2200));
        Keyboard.Listen(Key.Down, ButtonState.Down, LiikutaPelaajaaAlas, null, new Vector(0, -7000));
        
        AddCollisionHandler<PlatformCharacter, PhysicsObject>(goblin, LoytaaKolikon);
        AddCollisionHandler<PlatformCharacter, PhysicsObject>(goblin, LoytaaPortaalin);
        
        void PelaajaHyppaa(Vector vektori)
        {
            goblin.Jump(110);
        }
        
        void LiikutaPelaajaaOikealle(Vector vektori)
        {
            goblin.Walk(80);
        }
        
        void LiikutaPelaajaaVasemmalle(Vector vektori)
        {
            goblin.Walk(-80);
        }
        
        void LiikutaPelaajaaAlas(Vector vektori)
        {
            goblin.Push(vektori);
        }
    }

    private void LoytaaKolikon(PlatformCharacter goblin, PhysicsObject kohde)
    {
        if (kohde.Tag.ToString() == "kolikko")
        {
            AddCollisionHandler(kohde, goblin, CollisionHandler.DestroyObject);
            KeratytKolikot.Value += 1;
        }
    }
    
    private void LoytaaPortaalin(PlatformCharacter goblin, PhysicsObject kohde)
    {
        if (kohde.Tag.ToString() == "Maali")
        {
            KeratytKolikot.MaxValue = 6;
            MessageDisplay.Add("Voitit Pelin");
            goblin.Destroy();
        }
    }
    
    private void LuoPiikkeja(Vector paikka, double leveys, double korkeus, Color vari)
        {
            PhysicsObject piikkeja = new PhysicsObject(leveys, 30);
            piikkeja.Position = paikka;
            Image piikkienkuva = LoadImage("piikkeja");
            piikkeja.Image = piikkienkuva;
            piikkeja.Tag = "piikkeja";
            piikkeja.IgnoresPhysicsLogics = true;
            piikkeja.Mass = 100000;
            Add(piikkeja);

        } 
    
    private void LuoMaali(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject Portaali = new PhysicsObject(leveys , korkeus / 3);
        Portaali.Position = paikka;
        Portaali.Color = vari;
        Portaali.Tag = "Maali";
        Portaali.Image = LoadImage("Portaali");
        Add(Portaali);
    }
    
    IntMeter KeratytKolikot;
    
    private void LuoPistelaskuri(Vector paikka, double leveys, double korkeus, Color vari)
    {
        KeratytKolikot = new IntMeter(0);
        KeratytKolikot.MaxValue = 6;
        KeratytKolikot.UpperLimit += KaikkiKeratty;
        
        void KaikkiKeratty()
        {
            MessageDisplay.Add("Voit nyt kulkea Portaalista");
        }
        
        Label pistenaytto = new Label(); 
        pistenaytto.X = Screen.Left + 100;
        pistenaytto.Y = Screen.Top - 100;
        pistenaytto.TextColor = Color.Black;
        pistenaytto.Color = Color.Silver;
        pistenaytto.Title = "Kolikot:";

        pistenaytto.BindTo(KeratytKolikot);
        Add(pistenaytto);
    }
    
}
